

// Digital Material Cube 007
// FabLab Japan
// Hiroya Tanaka


// Import Mesh Library 
import toxi.geom.*;
import toxi.geom.mesh.*;
import toxi.geom.mesh.subdiv.*;
import toxi.processing.*;
import toxi.util.*;
import toxi.volume.*;
ToxiclibsSupport gfx;
WETriangleMesh mesh;

boolean mesh_display = true;
boolean voxel_display = true;
boolean wire_display = true;

// Import
import processing.opengl.*;
import javax.media.opengl.*;  
import controlP5.*;
import javax.swing.*;
import SpringGUI.*;
SpringGUI gui; 

// File IO
String getFile; 
String setFile;
String [] txtarray = null;
String txt = null;

// Field 
int FIELD_SIZE =  50;
int FIELD_STEP =  10;

// Camera 
float expo = 1;
float rotx, roty;
float rate = 0.01;

// Resolution of Box
int resolution = 1;

// ControlP5
ControlP5 controlP5;
Textfield tfUserName;       // テキストフィールド
String txts = "";

// Text
PFont  font;  
long   time;  

// Unlekker
import unlekker.data.*;
import unlekker.geom.*;
STL stl;
FaceList poly;
int faces; // メッシュの数

// Array 
int[][] xyarrays = new int[48][48];
int[][] yzarrays = new int[48][48];
int[][] zxarrays = new int[48][48];
int[][][] xyzarrays = new int[48][48][48];


void setup() {
  
  // Array初期化
    for(int k = 0; k<48; k = k + 1){
    for(int j = 0; j<48; j = j + 1){
    for(int i = 0; i<48; i = i + 1){
            xyzarrays[k][j][i] = 0; 
    }
    }
    }
    
    
  size(640, 480, OPENGL);    
  gfx=new ToxiclibsSupport(this);
  controlP5 = new ControlP5(this);  
  controlP5.addButton("loadfile",1,10,45,80,19);
  controlP5.addButton("savefile",1,100,45,80,19);
  controlP5.addButton("voxelize",1,190,45,80,19);
  controlP5.addSlider("resolution",1,48,10,70,200,20);
  
  controlP5.addToggle("mesh_display",true,300,45,20,10).setMode(ControlP5.SWITCH);
  controlP5.addToggle("voxel_display",true,400,45,20,10).setMode(ControlP5.SWITCH);
  controlP5.addToggle("wire_display",true,500,45,20,10).setMode(ControlP5.SWITCH);
}


void draw() {
  
  System.gc();
  
  background(0);
 // PGraphicsOpenGL pgl = (PGraphicsOpenGL)g;  

  
  // Lighting
    ambientLight(150, 150, 150);
    directionalLight(255,255,255,-1,0,0);
    pointLight(160, 160, 160, mouseX, mouseY, 200);
    spotLight(100, 100, 100, mouseX, mouseY, 200, 0, 0, -1, PI, 2);
    
  // Camera
  if (getFile == null && setFile == null) {
    if (mousePressed){
  rotx = rotx + (mouseY - pmouseY) * rate;
  roty = roty + (mouseX - pmouseX) * rate; 
    }
  } else { 
  rotx = 0;
  roty = 0;
  expo = 1;
  }
  
    
  // FileIO
    if(getFile != null){
    //ファイルを取り込む
    fileLoader(); 
  } 
  
  //Text
  font = createFont("Meiryo", 12);
  textFont(font);
  fill(255,255,255);
  text("07  VOXELIZE in AABB", 10, 25); 
  
 // Zoom In and Out
if (keyPressed && keyCode==UP) { expo = expo + 0.01; }
if (keyPressed && keyCode==DOWN) { expo = expo - 0.01; }

// 中心を合わせる
  pushMatrix(); 
  translate(320,240,200);
  rotateX(rotx); rotateY(roty); scale(expo);  
    
    

 
  // フレームの描画
  

  stroke(255, 255, 255);  
  FIELD_STEP = FIELD_SIZE*2/resolution;
  
  for(int k = 1; k<=resolution; k = k + 1){
    for(int j = 1; j<=resolution; j = j + 1){
       for(int i = 1; i<=resolution; i = i + 1){
       
          // x
          float x = -FIELD_SIZE + FIELD_STEP*(i - 1);
          float xx = -FIELD_SIZE + FIELD_STEP*(i);
          float realx = (x + xx)/2; 
          // y
          float y = -FIELD_SIZE + FIELD_STEP*(j - 1);
          float yy = -FIELD_SIZE + FIELD_STEP*(j);
          float realy = (y + yy)/2; 
          // z
          float z = -FIELD_SIZE + FIELD_STEP*(k - 1);
          float zz = -FIELD_SIZE + FIELD_STEP*(k);
          float realz = (z + zz)/2; 
        
        pushMatrix(); 
        translate(realx,realy,realz);
        noFill();
        if (wire_display == true){
        box(FIELD_STEP ,FIELD_STEP ,FIELD_STEP );
        }
        popMatrix();
       }
    }
  }

  

    // STL書き出し
    if(setFile != null){
   beginRaw("unlekker.data.STL",setFile+".stl");
   println("start :"+setFile);
  }
  
  // メッシュの描画

  
  if(poly!=null && mesh_display == true) {
    fill(255,255,255);
    noStroke();
    poly.draw(this);
  }
  

  
    // ボクセルの描画  

  noStroke();
  noFill();
  int len = txts.length(); 
  int l = 0;
  
  FIELD_STEP = FIELD_SIZE*2/resolution;

  for(int k = 1; k<=resolution; k = k + 1){
    for(int j = 1; j<=resolution; j = j + 1){
       for(int i = 1; i<=resolution; i = i + 1){
       
          //x
          float x = -FIELD_SIZE + FIELD_STEP*(i - 1);
          float xx = -FIELD_SIZE + FIELD_STEP*(i);
          float realx = (x + xx)/2; 
          //y
          float y = -FIELD_SIZE + FIELD_STEP*(j - 1);
          float yy = -FIELD_SIZE + FIELD_STEP*(j);
          float realy = (y + yy)/2; 
          //z
          float z = -FIELD_SIZE + FIELD_STEP*(k - 1);
          float zz = -FIELD_SIZE + FIELD_STEP*(k);
          float realz = (z + zz)/2; 
        
         pushMatrix();
         translate(realx,realy,realz); 
         if(  xyzarrays[i-1][j-1][k-1] == 1 && voxel_display == true )  { fill(200,00,00); box(FIELD_STEP,FIELD_STEP,FIELD_STEP); }

            
         popMatrix();
       }
    }
  }
  popMatrix();
  
   // STL書き出し終了
  if(setFile !=null) {
  endRaw();   
 setFile = null;
   println("end");
  }
 
 controlP5.draw();
 
}


// テキストが入力された際にコールバックされるメソッド
public synchronized void textfield(String txt) {
    txts = txt; 
}


public void loadfile(int theValue) {
    getFile = getFileName();
}


public void savefile(int theValue) {
     setFile = setFileName();
}

public void voxelize(int theValue) {
  
  // メッシュの数：int faces　　
  // AABB * AABBの衝突判定をつかってみる
  

  FIELD_STEP = FIELD_SIZE*2/resolution;
  
  for(int k = 1; k<=resolution; k = k + 1){
    for(int j = 1; j<=resolution; j = j + 1){
       for(int i = 1; i<=resolution; i = i + 1){
       
          // x
          float x = -FIELD_SIZE + FIELD_STEP*(i - 1);
          float xx = -FIELD_SIZE + FIELD_STEP*(i);
        
          // y
          float y = -FIELD_SIZE + FIELD_STEP*(j - 1);
          float yy = -FIELD_SIZE + FIELD_STEP*(j);
       
          //z
          float z = -FIELD_SIZE + FIELD_STEP*(k - 1);
          float zz = -FIELD_SIZE + FIELD_STEP*(k);

        
          

          for(int q = 0; q< poly.f.length; q= q + 1){
            
            boolean result = AABB( x, y, z, xx, yy, zz,  
            min(poly.f[q].v[3],poly.f[q].v[6],poly.f[q].v[9]), min(poly.f[q].v[4],poly.f[q].v[7],poly.f[q].v[10]), min(poly.f[q].v[5],poly.f[q].v[8],poly.f[q].v[11]),
            max(poly.f[q].v[3],poly.f[q].v[6],poly.f[q].v[9]), max(poly.f[q].v[4],poly.f[q].v[7],poly.f[q].v[10]), max(poly.f[q].v[5],poly.f[q].v[8],poly.f[q].v[11])); 
            //println(i + " ... "+ j + " ... " + k + " ... " + q + " = " + result);
         if (result == true) {      
             xyzarrays[i-1][j-1][k-1] = 1;  break;
         } else { 
             xyzarrays[i-1][j-1][k-1] = 0;  
         }
                
          }
       }
    }
  }
}

// AABBどうちの接触を調べる

public boolean AABB(float aminx,float aminy,float aminz, float amaxx,float amaxy, float amaxz, float bminx, float bminy, float bminz, float bmaxx, float bmaxy, float bmaxz) {
  // 衝突	
  if(              aminx < bmaxx && amaxx > bminx
		&& aminy < bmaxy && amaxy > bminy
		&& aminz < bmaxz && amaxz > bminz)
	{
                //println("1a");
		return(true);
	}
  if(              bminx < amaxx && bmaxx > aminx
		&& bminy < amaxy && bmaxy > aminy
		&& bminz < amaxz && bmaxz > aminz)
	{
               // println("1b");
		return(true);
	}

  // 包含
  if(              aminx < bminx && bmaxx < amaxx
		&& aminy < bminy && bmaxy < amaxy
		&& aminz < bminz && bmaxz < amaxz)
	{
		//println("2");
                return(true);
	}
  if(              bminx < aminx && amaxx < bmaxx
		&& bminy < aminy && amaxy < bmaxy
		&& bminz < aminz && amaxz < bmaxz)
	{
		//println("3");
                return(true);
	}
	return(false);
}

// 座標 p1,p2 を通る直線と座標 p3,p4 を結ぶ線分が交差しているかを調べる
public boolean intersection(float x1, float y1, float x2, float y2, float x3, float y3, float x4, float y4){
       
    if (((x1 - x2) * (y3 - y1) + (y1 - y2) * (x1 - x3)) * ((x1 - x2) * (y4 - y1) + (y1 - y2) * (x1 - x4)) <= 0 &&
        ((x3 - x4) * (y1 - y3) + (y3 - y4) * (x3 - x1)) * ((x3 - x4) * (y2 - y3) + (y3 - y4) * (x3 - x2)) <= 0 ) {
    
            return(true); //交差する
            
            
    } 
    
    return(false); //交差しない
  
    }
    
